@@ -88,7 +88,70 @@ char_u	digraphdefault[][3] = 		/* standard MSDOS digraphs */
 		};
 
 #else	/* !MSDOS && !WIN32 */
-# ifdef _INCLUDE_HPUX_SOURCE
+# ifdef MINT
+char_u	digraphdefault[][3] = 		/* standard ATARI digraphs */
+	   {{'C', ',', 128},	/* ~@ */
+		{'u', '"', 129},	/*  */
+		{'e', '\'', 130},	/*  */
+		{'a', '^', 131},	/*  */
+		{'a', '"', 132},	/*  */
+		{'a', '`', 133},	/*  */
+		{'a', '@', 134},	/*  */
+		{'c', ',', 135},	/* ~G */
+		{'e', '^', 136},	/* ~H */
+		{'e', '"', 137},	/*  */
+		{'e', '`', 138},	/*  */
+		{'i', '"', 139},	/*  */
+		{'i', '^', 140},	/*  */
+		{'i', '`', 141},	/*  */
+		{'A', '"', 142},	/*  */
+		{'A', '@', 143},	/*  */
+		{'E', '\'', 144},	/*  */
+		{'a', 'e', 145},	/*  */
+		{'A', 'E', 146},	/*  */
+		{'o', '^', 147},	/*  */
+		{'o', '"', 148},	/*  */
+		{'o', '`', 149},	/*  */
+		{'u', '^', 150},	/*  */
+		{'u', '`', 151},	/*  */
+		{'y', '"', 152},	/*  */
+		{'O', '"', 153},	/*  */
+		{'U', '"', 154},	/*  */
+	   	{'c', '|', 155},	/*  */
+	   	{'$', '$', 156},	/*  */
+	   	{'Y', '-', 157},	/* ~] */
+	   	{'s', 's', 158},	/*  */
+	    {'f', 'f', 159},	/*  */
+		{'a', '\'', 160},	/*  */
+		{'i', '\'', 161},	/*  */
+		{'o', '\'', 162},	/*  */
+		{'u', '\'', 163},	/*  */
+		{'n', '~', 164},	/*  */
+		{'N', '~', 165},	/*  */
+		{'a', 'a', 166},	/*  */
+		{'o', 'o', 167},	/*  */
+		{'~', '?', 168},	/*  */
+		{'-', 'a', 169},	/*  */
+		{'a', '-', 170},	/*  */
+		{'1', '2', 171},	/*  */
+		{'1', '4', 172},	/*  */
+		{'~', '!', 173},	/*  */
+		{'<', '<', 174},	/*  */
+		{'>', '>', 175},	/*  */
+		{'j', 'u', 230},	/*  */
+		{'o', '/', 237},	/*  */
+		{'+', '-', 241},	/*  */
+		{'>', '=', 242},	/*  */
+		{'<', '=', 243},	/*  */
+		{':', '-', 246},	/*  */
+		{'~', '~', 247},	/*  */
+		{'~', 'o', 248},	/*  */
+		{'2', '2', 253},	/*  */
+		{NUL, NUL, NUL}
+		};
+
+# else	/* !MINT */
+#  ifdef _INCLUDE_HPUX_SOURCE
 
 char_u	digraphdefault[][3] = 		/* default HPUX digraphs */
 	   {{'A', '`', 161},	/*  */
@@ -189,7 +252,7 @@ char_u	digraphdefault[][3] = 		/* default HPUX digraphs */
 		{NUL, NUL, NUL}
 		};
 
-# else	/* _INCLUDE_HPUX_SOURCE */
+#  else	/* _INCLUDE_HPUX_SOURCE */
 
 char_u	digraphdefault[][3] = 		/* standard ISO digraphs */
 	   {{'~', '!', 161},	/*  */
@@ -290,7 +353,8 @@ char_u	digraphdefault[][3] = 		/* standard ISO digraphs */
 		{NUL, NUL, NUL}
 		};
 
-# endif	/* _INCLUDE_HPUX_SOURCE */
+#  endif	/* _INCLUDE_HPUX_SOURCE */
+# endif	/* !MINT */
 #endif	/* !MSDOS && !WIN32 */
  
 /*
