@@ -1616,7 +1616,7 @@ ml_get_buf(buf, lnum, will_change)
 
 	if (lnum > buf->b_ml.ml_line_count)	/* invalid line number */
 	{
-		emsg2((char_u *)"ml_get: invalid lnum: %ld", (char_u *)lnum);
+		EMSGN("ml_get: invalid lnum: %ld", lnum);
 errorret:
 		STRCPY(IObuff, "???");
 		return IObuff;
@@ -1642,7 +1642,7 @@ errorret:
 		 */
 		if ((hp = ml_find_line(buf, lnum, ML_FIND)) == NULL)
 		{
-			emsg2((char_u *)"ml_get: cannot find line %ld", (char_u *)lnum);
+			EMSGN("ml_get: cannot find line %ld", lnum);
 			goto errorret;
 		}
 
@@ -2504,7 +2504,7 @@ ml_flush_line(buf)
 
 		hp = ml_find_line(buf, lnum, ML_FIND);
 		if (hp == NULL)
-			EMSG2("Cannot find line %ld", (char_u *)lnum);
+			EMSGN("Cannot find line %ld", lnum);
 		else
 		{
 			dp = (DATA_BL *)(hp->bh_data);
@@ -2783,10 +2783,11 @@ ml_find_line(buf, lnum, action)
 		if (idx >= (int)pp->pb_count)		/* past the end: something wrong! */
 		{
 			if (lnum > buf->b_ml.ml_line_count)
-				emsg2((char_u *)"line number out of range: %ld past the end", (char_u *)(lnum - buf->b_ml.ml_line_count));
+				EMSGN("line number out of range: %ld past the end",
+											  lnum - buf->b_ml.ml_line_count);
 
 			else
-				emsg2((char_u *)"line count wrong in block %ld", (char_u *)(bnum));
+				EMSGN("line count wrong in block %ld", bnum);
 			goto error_block;
 		}
 		if (action == ML_DELETE)
